'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

require('source-map-support/register');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

process.on('unhandledRejection', function (reason, p) {
    console.log('Unhandled Rejection at: Promise', p, 'reason:', reason);
    // application specific logging, throwing an error, or other logic here
});

console.log('cold start');
var AWS = require('aws-sdk');
var HTTPS = require('https');
var PROXY_AGENT = require('https-proxy-agent');

AWS.config.update({ region: process.env.GTM_AWS_REGION });

var zlib = require('zlib');

function handler(event, context, callback) {
    var _this = this;

    if (process.env.IAM_ENABLED) {
        AWS.config.update({
            httpOptions: {
                agent: PROXY_AGENT(process.env.AWS_PROXY)
            }
        });
    }

    var dynamo = void 0;
    if (process.env.GTM_DYNAMO_VPCE) {
        console.log('Configuring DynamoDB to use VPC Endpoint');
        dynamo = new AWS.DynamoDB({
            httpOptions: {
                agent: new HTTPS.Agent()
            }
        });
    } else {
        console.log('Configuring DynamoDB to use Global AWS Config');
        dynamo = new AWS.DynamoDB();
    }

    var ddb = new AWS.DynamoDB.DocumentClient({
        convertEmptyValues: true,
        service: dynamo
    });

    console.log('Recording Metrics...');

    var payload = new Buffer(event.awslogs.data, 'base64');
    zlib.gunzip(payload, function (err, res) {
        if (err) {
            console.log(err);
            return callback(err);
        }
        var payload = JSON.parse(res.toString('utf8'));
        console.log('Decoded Payload:', (0, _stringify2.default)(payload));

        var EVENTS_TABLE = process.env.GTM_DYNAMO_TABLE_EVENTS;
        var AGENTS_TABLE = process.env.GTM_DYNAMO_TABLE_AGENTS;

        var promises = [];

        payload.logEvents.forEach(function () {
            var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(evt) {
                var msg, isObj, updateParams, task;
                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                msg = void 0;
                                isObj = true;

                                try {
                                    msg = JSON.parse(evt.message);
                                } catch (e) {
                                    msg = evt.message;
                                    console.error('Unable to Parse Event Payload: ' + msg);
                                    isObj = false;
                                }

                                /***
                                 * Capture event summary data
                                 * gtmEvents table schema: ghEventId, startTime, eventDuration, endTime,
                                 * tasks, eventResult, repo, eventUrl, pullTitle, pullNumber, sha, eventUser
                                 */
                                if (isObj) {
                                    updateParams = void 0;


                                    if (msg.resultType === 'START') {
                                        console.log('Recording Event Start: ' + msg.ghEventId + '..');
                                        updateParams = {
                                            TableName: EVENTS_TABLE,
                                            Key: {
                                                ghEventId: msg.ghEventId
                                            },
                                            UpdateExpression: 'set startTime = :startTime, repo = :repo, eventUrl = :eventUrl, tasks = :tasks, ' + 'endTime = :endTime, eventDuration = :eventDuration, failed = :failed, ' + 'pullTitle = :pullTitle, pullNumber = :pullNumber, sha = :sha, eventUser = :eventUser, ' + 'agentId = :agentId, eventType = :eventType',
                                            ConditionExpression: 'attribute_not_exists(ghEventId) OR ghEventId = :ghEventId',
                                            ExpressionAttributeValues: {
                                                ':ghEventId': msg.ghEventId,
                                                ':startTime': msg.time,
                                                ':repo': msg.repo,
                                                ':eventUrl': msg.url,
                                                ':tasks': [],
                                                ':endTime': '',
                                                ':eventDuration': '',
                                                ':failed': '',
                                                ':pullTitle': msg.pullTitle,
                                                ':pullNumber': msg.pullNumber,
                                                ':sha': msg.sha,
                                                ':eventUser': msg.eventUser,
                                                ':agentId': msg.agentId,
                                                ':eventType': msg.eventType
                                            },
                                            ReturnValues: 'UPDATED_NEW'
                                        };
                                    }

                                    if (msg.resultType === 'AGENT_START') {
                                        console.log('Recording Agent Start: ' + msg.agentId + '..');
                                        updateParams = {
                                            TableName: AGENTS_TABLE,
                                            Key: {
                                                agentId: msg.agentId
                                            },
                                            UpdateExpression: 'set agentGroup = :agentGroup, startTime = :startTime, ' + 'version = :version, details = :details',
                                            ConditionExpression: 'attribute_not_exists(agentId) OR agentId = :agentId',
                                            ExpressionAttributeValues: {
                                                ':agentId': msg.agentId,
                                                ':startTime': msg.time,
                                                ':agentGroup': msg.agentGroup,
                                                ':version': msg.version,
                                                ':details': msg.details
                                            },
                                            ReturnValues: 'UPDATED_NEW'
                                        };
                                    }

                                    if (msg.resultType === 'TASK') {
                                        console.log('Recording Event Progress: ' + msg.ghEventId + '..');

                                        task = {
                                            executor: msg.executor,
                                            context: msg.context,
                                            duration: msg.duration,
                                            failed: msg.failed,
                                            time: msg.time
                                        };


                                        updateParams = {
                                            TableName: EVENTS_TABLE,
                                            Key: {
                                                ghEventId: msg.ghEventId
                                            },
                                            UpdateExpression: 'set tasks = list_append(tasks, :task)',
                                            ConditionExpression: 'attribute_not_exists(ghEventId) OR ghEventId = :ghEventId',
                                            ExpressionAttributeValues: {
                                                ':ghEventId': msg.ghEventId,
                                                ':task': [task]
                                            },
                                            ReturnValues: 'UPDATED_NEW'
                                        };
                                    }

                                    if (msg.resultType === 'EVENT') {
                                        console.log('Recording Event Completion: ' + msg.ghEventId + '..');

                                        updateParams = {
                                            TableName: EVENTS_TABLE,
                                            Key: {
                                                ghEventId: msg.ghEventId
                                            },
                                            UpdateExpression: 'set endTime = :endTime, eventDuration = :eventDuration, failed = :failed',
                                            ConditionExpression: 'attribute_not_exists(ghEventId) OR ghEventId = :ghEventId',
                                            ExpressionAttributeValues: {
                                                ':ghEventId': msg.ghEventId,
                                                ':endTime': msg.time,
                                                ':eventDuration': msg.duration,
                                                ':failed': msg.failed
                                            },
                                            ReturnValues: 'UPDATED_NEW'
                                        };
                                    }

                                    if (updateParams) {
                                        promises.push(ddb.update(updateParams).promise());
                                    }
                                }

                            case 4:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, _this);
            }));

            return function (_x) {
                return _ref.apply(this, arguments);
            };
        }());

        _promise2.default.all(promises).then(function () {
            var message = 'Successfully processed ' + payload.logEvents.length + ' log events.';
            console.log(message);
            callback(null, message);
        });
    });
}

module.exports = {
    handler: handler
};